@@ -1,5 +1,4 @@
 import numpy as np
-import sys
 
 from matplotlib import pyplot as plt, gridspec, patches, rc_context, cm, colors
 from astropy.wcs import WCS
@@ -10,7 +9,7 @@ from collections import Sequence
 from pathlib import Path
 
 from .model import c_KMpS, gaussian_profiles_computation, linear_continuum_computation, format_line_mask_option
-from .tools import label_decomposition, blended_label_from_log, UNITS_LATEX_DICT, latex_science_float
+from .tools import label_decomposition, blended_label_from_log, ASTRO_UNITS_KEYS, UNITS_LATEX_DICT, latex_science_float
 
 try:
     import pylatex
@@ -179,7 +178,7 @@ def numberStringFormat(value, cifras=4):
     return newFormat
 
 
-def mplcursors_legend(line, log, latex_label, norm_flux, units_wave):
+def mplcursors_legend(line, log, latex_label, norm_flux, units_wave, units_flux):
 
     if len(latex_label) == 1:
         legend_text = latex_label[0] + '\n'
@@ -191,22 +190,25 @@ def mplcursors_legend(line, log, latex_label, norm_flux, units_wave):
         ion, wave, latex = label_decomposition(line, scalar_output=True, units_wave=units_wave)
         legend_text = latex + '\n'
 
+    units_line_flux = ASTRO_UNITS_KEYS[units_wave] * ASTRO_UNITS_KEYS[units_flux]
+    line_flux_latex = f'{units_line_flux:latex}'
+    normFlux_latex = f' $({latex_science_float(norm_flux)})$' if norm_flux != 1 else ''
+
     intg_flux = latex_science_float(log.loc[line, 'intg_flux']/norm_flux)
     intg_err = latex_science_float(log.loc[line, 'intg_err']/norm_flux)
-    normFlux = latex_science_float(norm_flux)
-    legend_text += r'$F_{{intg}} = {}\pm{}\,({})$'.format(intg_flux, intg_err, normFlux) + '\n'
+    legend_text += r'$F_{{intg}} = {}\pm{}\,$'.format(intg_flux, intg_err) + line_flux_latex + normFlux_latex + '\n'
 
     gauss_flux = latex_science_float(log.loc[line, 'gauss_flux']/norm_flux)
     gauss_err = latex_science_float(log.loc[line, 'gauss_err']/norm_flux)
-    legend_text += r'$F_{{gauss}} = {}\pm{}\,({})$'.format(gauss_flux, gauss_err, normFlux) + '\n'
+    legend_text += r'$F_{{gauss}} = {}\pm{}\,$'.format(gauss_flux, gauss_err) + line_flux_latex + normFlux_latex + '\n'
 
     v_r = r'{:.1f}'.format(log.loc[line, 'v_r'])
     v_r_err = r'{:.1f}'.format(log.loc[line, 'v_r_err'])
-    legend_text += r'$v_{{r}} = {}\pm{}\,(km/s)$'.format(v_r, v_r_err) + '\n'
+    legend_text += r'$v_{{r}} = {}\pm{}\,\frac{{km}}{{s}}$'.format(v_r, v_r_err) + '\n'
 
     sigma_vel = r'{:.1f}'.format(log.loc[line, 'sigma_vel'])
     sigma_vel_err = r'{:.1f}'.format(log.loc[line, 'sigma_vel_err'])
-    legend_text += r'$\sigma_{{g}} = {}\pm{}\,(km/s)$'.format(sigma_vel, sigma_vel_err)
+    legend_text += r'$\sigma_{{g}} = {}\pm{}\,\frac{{km}}{{s}}$'.format(sigma_vel, sigma_vel_err)
 
     return legend_text
 
@@ -456,7 +458,7 @@ class LiMePlots:
         return
 
     def plot_spectrum(self, comp_array=None, peaks_table=None, match_log=None, noise_region=None,
-                      log_scale=False, plt_cfg={}, ax_cfg={}, spec_label='spectrum', output_address=None,
+                      log_scale=False, plt_cfg={}, ax_cfg={}, spec_label='Object spectrum', output_address=None,
                       include_fits=False, log=None, frame='observed'):
 
         """
@@ -544,15 +546,26 @@ class LiMePlots:
                 ax.step(wave_plot/z_corr, comp_array, label='Sigma Continuum', linestyle=':', where='mid')
 
             # Plot peaks and troughs if provided
-            if peaks_table is not None:
+            if (peaks_table is not None) or (match_log is not None):
+
                 color_peaks = (self._color_dict['peak'], self._color_dict['trough'])
+                labels = ('Peaks', 'Troughs')
                 line_types = ('emission', 'absorption')
                 labels = ('Peaks', 'Troughs')
-                for i in range(2):
-                    idcs_emission = peaks_table['line_type'] == line_types[i]
-                    idcs_linePeaks = np.array(peaks_table[idcs_emission]['line_center_index'])
-                    ax.scatter(wave_plot[idcs_linePeaks]/z_corr, flux_plot[idcs_linePeaks]*z_corr, label=labels[i],
-                               facecolors='none', edgecolors=color_peaks[i])
+
+                if peaks_table is not None:
+                    line_types = ('emission', 'absorption')
+                    for i in range(2):
+                        idcs_emission = peaks_table['line_type'] == line_types[i]
+                        idcs_linePeaks = np.array(peaks_table[idcs_emission]['line_center_index'])
+                        ax.scatter(wave_plot[idcs_linePeaks]/z_corr, flux_plot[idcs_linePeaks]*z_corr, label=labels[i],
+                                   facecolors='none', edgecolors=color_peaks[i])
+
+                else:
+                    if 'signal_peak' in match_log:
+                        idcs_linePeaks = match_log['signal_peak'].values.astype(int)
+                        ax.scatter(wave_plot[idcs_linePeaks] / z_corr, flux_plot[idcs_linePeaks] * z_corr, label='Peaks',
+                                   facecolors='none', edgecolors=self._color_dict['peak'])
 
             # Shade regions of matched lines if provided
             if match_log is not None:
@@ -635,6 +648,12 @@ class LiMePlots:
                 for label, lineProfile in self._legends_dict.items():
                     mplcursors.cursor(lineProfile).connect("add", lambda sel, label=label: sel.annotation.set_text(label))
 
+            # Plot the masked pixels
+            if self._masked_inputs:
+                idcs_mask = self.flux.mask
+                ax.scatter(wave_plot[idcs_mask]/z_corr, flux_plot[idcs_mask]*z_corr, marker='x', label='Masked pixels',
+                           color=self._color_dict['mask_marker'])
+
             # Switch y_axis to logarithmic scale if requested
             if log_scale:
                 ax.set_yscale('log')
@@ -722,7 +741,7 @@ class LiMePlots:
 
             # Plot the gauss curve elements
             self.gaussian_profiles_plotting(list_comps, self.log, wave_plot[idcs_plot], flux_plot[idcs_plot], z_corr,
-                                            axis=spec_ax, frame=frame, peak_check=True, cont_bands=True,
+                                            axis=spec_ax, frame=frame, peak_check=False, cont_bands=True,
                                             wave_array=wave_array, cont_array=cont_array, gaussian_array=gaussian_array,
                                             mplcursors_active=True)
 
@@ -737,10 +756,11 @@ class LiMePlots:
             resid_ax.fill_between(wave_plot[idcs_plot]/z_corr, -y_limit, +y_limit, facecolor='yellow', alpha=0.5, label=label)
 
             # Marked masked pixels if they are there
-            if idcs_mask is not None:
+            if np.any(idcs_mask):
                 x_mask = wave_plot[idcs_plot][idcs_mask[idcs_plot]]
                 y_mask = flux_plot[idcs_plot][idcs_mask[idcs_plot]]
-                spec_ax.scatter(x_mask/z_corr, y_mask*z_corr, marker="x", color=self._color_dict['mask_marker'])
+                spec_ax.scatter(x_mask/z_corr, y_mask*z_corr, marker="x", color=self._color_dict['mask_marker'],
+                                label='Masked pixels')
 
             # Plot masked pixels if possible
             self.mask_pixels_plotting(list_comps[0], wave_plot, flux_plot, z_corr, spec_ax, self.log)
@@ -1009,10 +1029,9 @@ class LiMePlots:
         if np.ma.is_masked(wave_obs):
             idcs_mask = wave_obs.mask
             wave_plot, flux_plot = wave_obs.data, flux_obs.data
-            flux_plot[idcs_mask] = flux_plot[idcs_mask]/self.norm_flux
 
         else:
-            idcs_mask = None
+            idcs_mask = np.zeros(flux_obs.size).astype(bool)
             wave_plot, flux_plot = wave_obs, flux_obs
 
         return wave_plot, flux_plot, z_corr, idcs_mask
@@ -1073,7 +1092,7 @@ class LiMePlots:
 
                 # Compute mplcursors box text
                 if mplcursors_check:
-                    label_complex = mplcursors_legend(line, log, latex_array, self.norm_flux, self.units_wave)
+                    label_complex = mplcursors_legend(line, log, latex_array, self.norm_flux, self.units_wave, self.units_flux)
                     self._legends_dict[label_complex] = line_g
 
             # Combined profile if applicable
@@ -1103,6 +1122,87 @@ class LiMePlots:
 
         return
 
+    def _plot_continuum_fit(self, continuum_fit, idcs_cont, low_lim, high_lim, threshold_factor, plot_title=''):
+
+        PLOT_CONF = STANDARD_PLOT.copy()
+        AXES_CONF = STANDARD_AXES.copy()
+
+        norm_label = r' $\,/\,{}$'.format(latex_science_float(self.norm_flux)) if self.norm_flux != 1.0 else ''
+        AXES_CONF['ylabel'] = f'Flux $({UNITS_LATEX_DICT[self.units_flux]})$' + norm_label
+        AXES_CONF['xlabel'] = f'Wavelength $({UNITS_LATEX_DICT[self.units_wave]})$'
+        AXES_CONF['title'] = plot_title
+
+        wave_plot, flux_plot, z_corr, idcs_mask = self.frame_mask_switch(self.wave, self.flux, self.redshift, 'observed')
+
+        with rc_context(PLOT_CONF):
+
+            fig, ax = plt.subplots()
+
+            # Object spectrum
+            ax.step(wave_plot, flux_plot, label='Object spectrum', color=self._color_dict['fg'])
+
+            # Band limits
+            label = r'$16^{{th}}/{} - 84^{{th}}\cdot{}$ flux percentiles band'.format(threshold_factor, threshold_factor)
+            ax.axhspan(low_lim, high_lim, alpha=0.2, label=label, color=self._color_dict['line_band'])
+            ax.axhline(np.median(flux_plot[idcs_cont]), label='Median flux', linestyle=':', color='black')
+
+            # Masked and rectected pixels
+            ax.scatter(wave_plot[~idcs_cont], flux_plot[~idcs_cont], label='Rejected pixels',
+                       color=self._color_dict['peak'], facecolor='none')
+            ax.scatter(wave_plot[idcs_mask], flux_plot[idcs_mask], marker='x', label='Masked pixels',
+                       color=self._color_dict['mask_marker'])
+
+            # Output continuum
+            ax.plot(wave_plot, continuum_fit, label='Continuum')
+
+            ax.update(AXES_CONF)
+            ax.legend()
+            plt.tight_layout()
+            plt.show()
+
+        return
+
+    def _plot_peak_detection(self, peak_idcs, detect_limit, continuum=None, plot_title='', ml_mask=None):
+
+        PLOT_CONF = STANDARD_PLOT.copy()
+        AXES_CONF = STANDARD_AXES.copy()
+
+        norm_label = r' $\,/\,{}$'.format(latex_science_float(self.norm_flux)) if self.norm_flux != 1.0 else ''
+        AXES_CONF['ylabel'] = f'Flux $({UNITS_LATEX_DICT[self.units_flux]})$' + norm_label
+        AXES_CONF['xlabel'] = f'Wavelength $({UNITS_LATEX_DICT[self.units_wave]})$'
+        AXES_CONF['title'] = plot_title
+
+        wave_plot, flux_plot, z_corr, idcs_mask = self.frame_mask_switch(self.wave, self.flux, self.redshift, 'observed')
+
+        continuum = continuum if continuum is not None else np.zeros(self.flux.size)
+
+        with rc_context(PLOT_CONF):
+
+            fig, ax = plt.subplots()
+            ax.step(wave_plot, flux_plot, color=self._color_dict['fg'], label='Object spectrum')
+
+            if ml_mask is not None:
+                if np.any(ml_mask):
+                    ax.scatter(wave_plot[ml_mask], flux_plot[ml_mask], label='ML detection', color='palegreen')
+
+            ax.scatter(wave_plot[peak_idcs], flux_plot[peak_idcs], marker='o', label='Peaks', color=self._color_dict['peak'], facecolors='none')
+            ax.fill_between(wave_plot, continuum, detect_limit, facecolor=self._color_dict['line_band'], label='Noise_region', alpha=0.5)
+
+            if continuum is not None:
+                ax.plot(wave_plot, continuum, label='Continuum')
+
+            ax.scatter(wave_plot[idcs_mask], flux_plot[idcs_mask], label='Masked pixels', marker='x',
+                       color=self._color_dict['mask_marker'])
+
+
+
+            ax.legend()
+            ax.update(AXES_CONF)
+            plt.tight_layout()
+            plt.show()
+
+        return
+
 
 class PdfMaker:
 
